@@ -19,7 +19,10 @@ class TionDelegation(DefaultDelegate):
     def handleNotification(self, handle: int, data: bytes):
         self._data.append(data)
         _LOGGER.debug("Got data in %d response %s", handle, bytes(data).hex())
-        self.__topic.read()
+        try:
+            self.__topic.read()
+        except btle.BTLEDisconnectError as e:
+            _LOGGER.warning("Got %s while read in handleNotification. May continue working.", str(e))
 
     def setReadTopic(self, topic):
         self.__topic = topic
@@ -98,10 +101,10 @@ class tion(TionDummy):
         self._error_code: int = 0
         self.__failed_connects: int = 0
         self.__connections_count: int = 0
+        self.__notifications_enabled: bool = False
 
         if self.mac == "dummy":
             _LOGGER.warning("Dummy mode detected!")
-            self._dummy_data: bytearray = bytearray()
             self._connect = self._connect_dummy
             self._disconnect = self._disconnect_dummy
             self._try_write = self._try_write_dummy
@@ -326,6 +329,9 @@ class tion(TionDummy):
 
     def _disconnect(self):
         if self.connection_status != "disc":
+            if self.__notifications_enabled:
+                self._disable_notifications()
+
             self._btle.disconnect()
 
     def _try_write(self, request: bytearray):
@@ -357,16 +363,20 @@ class tion(TionDummy):
 
         return response
 
+    def __write_to_notify_handle(self, data):
+        _LOGGER.debug("Notify handler is %s", self.notify.getHandle())
+        notify_handle = self.notify.getHandle() + 1
+
+        _LOGGER.debug("Will write %s to %s handle", data, notify_handle)
+        result = self._btle.writeCharacteristic(notify_handle, data, withResponse=True)
+        _LOGGER.debug("Result is %s", result)
+
     def _enable_notifications(self):
         _LOGGER.debug("Enabling notification")
         setup_data = b"\x01\x00"
 
-        _LOGGER.debug("Notify handler is %s", self.notify.getHandle())
-        notify_handle = self.notify.getHandle() + 1
+        self.__write_to_notify_handle(setup_data)
 
-        _LOGGER.debug("Will write %s to %s handle", setup_data, notify_handle)
-        result = self._btle.writeCharacteristic(notify_handle, setup_data, withResponse=True)
-        _LOGGER.debug("Result is %s", result)
         self._btle.withDelegate(self._delegation)
         _LOGGER.debug("Delegation enabled")
         try:
@@ -378,6 +388,16 @@ class tion(TionDummy):
 
         self._delegation.setReadTopic(self.notify)
         _LOGGER.debug("enable_notification is done")
+        self.__notifications_enabled = True
+
+    def _disable_notifications(self):
+        _LOGGER.debug("Disabling notifications")
+        setup_data = b"\x00\x00"
+
+        self.__write_to_notify_handle(setup_data)
+
+        _LOGGER.debug("disable_notification is done")
+        self.__notifications_enabled = False
 
     @property
     def fan_speed(self):
